% function dat = scanRF( fp, RFs )
% scan RF response of a FP cavity.
% The main propagator is to be named as FPprop
% ETM is to be named as 'ETM'
function dat = scanRF( fp, RFs )

% go through RFs
% if RF is a single integer, make an array using FSR with step of FSR/RFs
if isnumeric(RFs) && length(RFs) == 1 && (floor(RFs) == RFs) && (RFs > 0)
    RFs = (0:RFs) / RFs * ConstantTK.c / 2 / fp.props.getPropLeng('FPprop');
end

% lock cavity
fp.lock;

% use the lock length and change the RF
for irf = 1 : length( RFs )
    fp.setRF( RFs(irf) );  % set RF
    fp.calc( 1e-5, 0);  % calculate with minimal output
    dat(irf,1) = RFs(irf);
    dat(irf,2) = fp.power('ETM-HR-o'); % total power
    dat(irf,3) = fp.roundtripLoss;
    dat(irf,4) = 1 - fp.HGFrac('ETM-HR-o', 0, 0); % HOM fraction
    dat(irf,5) = fp.HGFrac('ETM-HR-o', 1, 0); % HG10 power fraction
    dat(irf,6) = fp.LGFrac('ETM-HR-o', 2, 0); % LG20 fraction
    dat(irf,7) = fp.LGFrac('ETM-HR-o', 3, 0); % LG30 fraction
    dat(irf,8) = fp.LGFrac('ETM-HR-o', 4, 0); % LG40 fraction
    dat(irf,9) = fp.LGFrac('ETM-HR-o', 5, 0); % LG50 fraction
end
