function buildMetroProData( outFile, data, cameraRes, wavelength, templateFile )
% function buildMetroProData( outFile, data, cameraRes, wavelength, templateFile )

if ~exist( 'templateFile', 'var')
    templateFile = './data/MetroProFormatTemplate.dat';
end
if ~exist( 'wavelength', 'var' )
    wavelength = 1.064e-6;
end

if iscell(templateFile)
    templateFile = templateFile{1};
end

if iscell(outFile)
    outFile = outFile{1};
end


[status, result] = unix(sprintf('cp %s %s', templateFile, outFile));
if status
    error( result );
end

fid = fopen( outFile, 'r+', 'b' );
if fid == -1
    error('output file cannot be opened');
end;

[sizeY, sizeX] = size( data );
fseek( fid, 68, 'bof' ); 
fwrite( fid, sizeX, 'int16' );
fwrite( fid, sizeY, 'int16' );
fwrite( fid, sizeX*sizeY*4, 'int32' );

fseek( fid, 164, 'bof' );
intf_scale_factor = fread( fid, 1, 'float' );
fseek( fid, 168, 'bof' );
fwrite( fid, wavelength, 'float' );
fseek( fid, 176, 'bof' );
obliquity_factor = fread( fid, 1, 'float' );

fseek( fid, 184, 'bof' );
fwrite( fid, cameraRes, 'float' );

phaseResVal = 32768; % for format 0 and phaseRes = 1

convFactor = intf_scale_factor * obliquity_factor * wavelength / phaseResVal;
indNaN = find( isnan( data ) );
iData = int32( data / convFactor );
iData( indNaN ) = hex2dec('7FFFFFF8');

fseek( fid, 0, 'eof' );
count = fwrite( fid, fliplr(iData'), 'int32' );
if count ~= sizeX*sizeY
    error('could not write all data');
end
fclose( fid );

end
