function [amap, xl, yl] = loadSISData( fileName )

baseid='read_SIS_map';

amap = [];
NBegin = regexp( fileName, '_N\d');
WBegin = regexp( fileName, '_W\d');

% this is not SIS format file
if isempty( NBegin ) || isempty( WBegin )
    return;
end

% read file name to find the FFT dimension
Nfft = sscanf( fileName( NBegin+2:end ), '%d' );
Wfft = sscanf( fileName( WBegin+2:end ), '%d' );
unitBegin = regexp( fileName, '_W\d*', 'end') + 1;
unitName = fileName( unitBegin:unitBegin+1 );
switch lower(unitName)
    case 'cm'
        unitScale = 1e-2;
    case 'mm'
        unitScale = 1e-3;
    case 'um'
        unitScale = 1e-6;
    otherwise
        unitScale = 1e-3;
end
Wfft = Wfft * unitScale;

amap = load( fileName );
if isempty( amap )
    msgid=[baseid,':fileopen'];
    error(msgid,result);
end

% change the definition of (x,y) axis convention
amap = amap';
dx = Wfft / Nfft;
xl = -Wfft/2+dx/2 : dx : Wfft/2-dx/2;
yl = xl;

end


