
classdef BSsize < FFTIFO
    
    methods
        function defineIFO( obj )
            
            dPRC12 = 16.6107; dPRC23 = 16.1647; dPRC3B = 19.5384;
            dCP = 0.10;  dBSCPX = 4.8046;  dBSCPY = 4.8497; dCPITM = 0.005;
            thetaPR2 = 0.79/180*pi;  thetaPR3 = 0.615/180*pi;  
            
            rmloss = 1;
            ITMloss = 1;
            angleScale = 0;
            armScale = 1;
            armLoss = 35e-6 * armScale;

            Atm = 0.326;
            mscale = 1;
%            Atm = Atm * mscale;
            Apr3 = 0.262;
            Abs = 0.3671 * mscale;
            dBS = 0.06;
            AspRatio = 0.06 / 0.3671;
              Abs = 0.45;
              dBS = Abs * AspRatio;
% add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', -1/11.009, 'Thick', 0.0749, ...
                'T', 3.1e-2, 'Lhr', (15+0.5)*1e-6*rmloss, 'Lar', (11.45+26)*1e-6*rmloss );
            obj.addMirror( 'PRM2', 'invRoC', -1/4.545, ...
                'T', 243e-6*rmloss, 'Lhr', 8.6e-6*rmloss, 'inc', thetaPR2*angleScale );
            obj.addMirror( 'PRM3', 'invRoC', 1/36.027, 'Aperture', Apr3, ...
                'T', 5.3e-6*rmloss, 'Lhr', 17e-6*rmloss, 'incAngle', thetaPR3*angleScale );
            
            BSID = obj.addMirror( 'BS', 'Aperture', Abs, 'Thickness', dBS, ...
                'T', 0.5, 'Lhr', 8.6e-6*rmloss, 'Lar', 31.7e-6*rmloss, 'incAngle', pi/4 );
            ITMXID = obj.addMirror( 'ITMX', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', 0.2 + dCP, ...
                'T', 1.48e-2, 'Lhr', armLoss/2, 'Lar', (164+42)*1e-6*ITMloss);
            ITMYID = obj.addMirror( 'ITMY', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', 0.2 + dCP, ...
                'T', 1.48e-2, 'Lhr', armLoss/2, 'Lar', (250+80)*1e-6*ITMloss );
            
              obj.optics.optList(ITMXID).TRfiles.addMap( '', '-(x.^2+y.^2)/2/50e3*1.45', 0.16, 1e10);
              obj.optics.optList(ITMYID).TRfiles.addMap( '', '-(x.^2+y.^2)/2/50e3*1.45', 0.16, 1e10);
 
            obj.addDetector( 'dark' );
            
            % implement BS baffle
            obj.optics.optList(BSID).modMap = @obj.BSbaffle;

            % ETMX
            ETMXID = obj.addMirror( 'ETMX', 'invRoC', 1/2245, 'Aperture', Atm, 'Thickness', 0.2, ...
                'T', 3.7e-6 * armScale, 'Lhr', armLoss/2 );
            % ETMY
            ETMYID = obj.addMirror( 'ETMY', 'invRoC', 1/2245, 'Aperture', Atm, 'Thickness', 0.2, ...
                'T', 3.7e-6 * armScale, 'Lhr', armLoss/2 );
            
            % arms
            obj.props.addProp( '', 'ITMX',  1, 'ETMX', 1, 3994.5);
            obj.props.addProp( '', 'ITMY',  1, 'ETMY', 1, 3994.5);

            obj.props.addProp( '', 'src', 1, 'PRM', 2 );
            obj.props.addProp( '', 'PRM',  1, 'PRM2', 1, dPRC12 );
            obj.props.addProp( '', 'PRM2',  2, 'PRM3', 1, dPRC23 );
            obj.props.addProp( '', 'PRM3',  2, 'BS', 1, dPRC3B );
            
            obj.props.addProp( '', 'BS',  3, 'ITMX', 2, dBSCPX + dCPITM);
            obj.props.addProp( '', 'BS',  2, 'ITMY', 2, dBSCPY + dCPITM);
            
% make sure the following one to be commented out when SRC is added
            obj.props.addProp( '', 'BS',  4, 'dark', 1 );
        end
        

% this adjusts the PRM2-PRM3 length to make the PRC mode matches with the arm mode
%         function setupBaseMode( obj )
%             % adjust the PRM2-PRM3 length so that the PRC mode matches with the arm mode
%             ITMYHRoutID = obj.optics.findPortID( 'ITMY', 1 );
%             ITMYHRinID = obj.optics.findPortID( 'ITMY', -1 );
%             ETMYHRoutID = obj.optics.findPortID( 'ETMY', 1 );
%             ITMYARoutID = obj.optics.findPortID( 'ITMY', 2 );
%             BSHRxoutID = obj.optics.findPortID( 'BS', 1 );
%             PRM3xoutID = obj.optics.findPortID( 'PRM3', 1 );
%             PRM2xoutID = obj.optics.findPortID( 'PRM2', 1 );
%             
%             % resonating mode in the arm.  rootArmY is ITMYHRoutID
%             [qInvArmY, rootArmY] = obj.cavities.findCavityMode( [ITMYHRoutID, ETMYHRoutID] );
%             sprintf('Mode used to define the modes of the entire IFO : portID = %d, RoC = %g\n',...
%                 rootArmY, 1/real(qInvArmY) )
%             
%             % find the q value going out of the y arm reverse the q value going into the ITMY HR, then let it goes through ITMY toward AR side
%             qInvArmYAR = MMTK.ABCD_qinv( obj.optics.findABCD( ITMYHRinID, ITMYARoutID ), MMTK.reverseDir( qInvArmY ) );
%             
%             % find the adjustment of the PRM3-PRM2 length
%             LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
%                 [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
%             
%             sprintf('PR2-PR3 adjustment : %g\n', LadjP )
%             
%             obj.cavities.fillQvals( rootArmY, qInvArmY );
%         end

        function setupBaseMode( obj )
            ITMYARoutID = obj.optics.findPortID( 'ITMY', 2 );
            BSHRxoutID = obj.optics.findPortID( 'BS', 1 );
            PRM3xoutID = obj.optics.findPortID( 'PRM3', 1 );
            PRM2xoutID = obj.optics.findPortID( 'PRM2', 1 );

            qInvArmYAR = 1.45*(1/1934 - 1/(1.45*50e3)) - 1i*1.064e-6/pi/0.053^2;
            % find the adjustment of the PRM3-PRM2 length
            LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
                [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
            
            sprintf('PR2-PR3 adjustment : %g\n', LadjP )
            
            obj.cavities.fillQvals( ITMYARoutID, qInvArmYAR );
        end

      end
    
    methods( Static )
        % implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y )
            Ax = 0.260;
            Ay = 0.260;
            dx = 0.000;
            
%              Ax = 0.237;
%              Ay = 0.260;
%             dx = 0.002;

%            baffleMask = x.^2/0.105^2+y.^2/0.13^2 < 1;
            baffleMaskX = (x-dx).^2/(Ax/2)^2+y.^2/(Ay/2)^2 < 1;
            baffleMaskY = x.^2/(Ax/2)^2+y.^2/(Ay/2)^2 < 1;

            % BS HRx in -> BS HRy out
            reflOut{1} = baffleMaskY .* reflIn{1};
            % BS HRy in -> BS HRx out
            reflOut{2} = reflIn{2} .* fliplr(baffleMaskY);
            % BS ARx in -> BS ARy out
            reflOut{3} = reflIn{3} .* fliplr(baffleMaskX);
            % BS ARy in -> BS ARx out
            reflOut{4} = baffleMaskX .* reflIn{4};
            % BS HRx in -> BS ARx out
            transOut{1} = baffleMaskX .* transIn{1};
            % BS HRy in -> BS ARy out
            transOut{2} = transIn{2} .* fliplr(baffleMaskY);
            % BS ARx in -> BS HRx out
            transOut{3} = transIn{3} .* fliplr(baffleMaskX);
            % BS ARy in -> BS HRy out
            transOut{4} = baffleMaskY .* transIn{4};
        end
    end
end