%% create FP cavity based on FFTIFO
classdef FPIFO < FFTIFO
    
    methods
        function obj = FPIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end
        
        %% this defines the IFO configuration
        function defineIFO( obj, varargin )            
            loss = 0;
            
            %% add optics
            % laser source
            obj.addLaser( 'src' );
            % ITM
            dITM = 0.2;
            Aperture = 0.326;
            % coated mirror data
            ITMRoC = 1934;
            ETMRoC = 2245;
            
            ARMID = 1;
            ITMS1File = '';
            ETMS1File = '';
            useTWE = 0;
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            switch ARMID
                case 1  % LLO X arm                    
                    ITMRoC = 1937.9;
                    ETMRoC = 2239.7;
                    ITMS1File = './Data/ITM04_CITZygo.dat';
                    ETMS1File = './Data/ETM07_CIT.dat';
                case 11 % LLO X arm polished
% polished mirror data
%           ITMRoC = 1938.61;
%           ETMRoC = 2239.7;
                    ITMS1File = './Data/itm04_s1_ASML.dat';
                    ETMS1File = './Data/xETM_07_R1_Figure_ASML.dat';
                case 2  % LLO Y arm
                    % ITMRoC = 1940.7;
%           ETMRoC = 2242.4;
                    ITMS1File = './Data/ITM08_CITZygo.dat';
                    ETMS1File = './Data/ETM09_CIT.dat';
                case 3  % LHO X arm
                    % ITMRoC = 1939.3;
%           ETMRoC = 2241.54;
                    ITMS1File = './Data/ITM03_CIT.dat';
                    ETMS1File = './Data/ETM08_CIT.dat';
                case 4  % LHO X arm
                    % ITMRoC = 1939.20;
%                     ETMRoC = 2238.9;
                    ITMS1File = './Data/ITM11_CIT.dat';
                    ETMS1File = './Data/ETM12_CIT.dat';
            end
            
            obj.addMirror( 'ITM', 'invRoC', 1/ITMRoC, 'Aperture', Aperture, 'Thick', dITM, ...
                'T', 1.48e-2, 'Lhr', 10.4e-6 + loss/2, 'Lar', (164+42)*1e-6, 'removeTilt', true );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/ETMRoC, 'Aperture', Aperture, 'Thick', 0.2, ...
                'T', 3.7e-6, 'Lhr', 10.9e-6 + loss/2, 'resolution', 2e-3, 'removeTilt', true  );
            % detector behind ETM
            obj.addDetector( 'trans' );
            
            %% define connections among optics
            obj.addProp( '', 'src', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', 3994.5);
            obj.addProp( '', 'ETM-AR', 'trans' );
            
            %% modify the defintions of test masses
             obj.setHRfiles( 'ITM', ITMS1File, '', 0.16, ITMRoC );
             if useTWE
                 switch ARMID
                     case 1
                         obj.setTRfiles( 'ITM', './Data/itm04_sptwe.dat', '', 0.16, 302.41e3);
                         obj.setTRfiles( 'ITM', './Data/CP06_sptwe.dat', '', 0.16, 1e10);
                 end
             end
            % ETM HR surface
             obj.setHRfiles( 'ETM', ETMS1File, '', 0.16, ETMRoC );
             
        end
    end
end
