%% create FP cavity based on FFTIFO
classdef FPKAGRA < FFTIFO
    
    methods
        function obj = FPKAGRA( varargin )
            obj@FFTIFO( varargin{:} );
        end

        function defineIFO( obj, varargin )
            
            ip = inputParser;
            addParameter(ip, 'power',  1,  @(x)isnumeric(x) && x>0); % power of laser
            ip.parse(varargin{:})
            
            %% add optics
            %  help FFTIFO.addMirror lists all avairable parameters
            RoCITMX = 1.9e3;
            RoCETMX = 1.9e3;
            ATM = 0.35;
            dTM = 0.10;
            armLoss = 80e-6 * 0;
            
            % laser source
            obj.addLaser( 'src', ip.Results.power );
            % ITM
            obj.addMirror( 'ITMX', 'invRoC', 1/RoCITMX, 'Aperture', ATM, 'Thick', dTM, ...
                'T', 0.004, 'Lhr', armLoss/2 );
            % ETM
            obj.addMirror( 'ETMX', 'invRoC', 1/RoCETMX, 'Aperture', ATM, 'Thick', dTM, ...
                'T', 5e-6, 'Lhr', armLoss/2 );
            % detection port
            obj.addDetector( 'trans' );
            
            %% define connections among optics
            obj.addProp( '', 'src', 'ITMX-AR' );
            obj.addProp( 'FPprop', 'ITMX-HR', 'ETMX-HR', 3000);
            obj.addProp( '', 'ETMX-AR', 'trans' );
            
            %% modify the defintions of test masses
            %  help FFTIFO.setHRfiles explains arguments
            %  setHRfiles defines the height of HR surface.
            %  To add a concave curvature, use r^2/(2*RoC) with RoC > 0.
            obj.setHRfiles( 'ITMX', '', ...
                'calcThermal(0, r, PsubIX, PcoatIX, wIX, 0.1, 0.175 ) + r.^2/2*invRoCITMXRH', ...
                0.16, RoCITMX, 0, 'PsubIX', 0', 'PcoatIX', 0, 'wIX', 0.0353, 'invRoCITMXRH', 0 );
            
            %  setTRfiles defines the transmission map, which is negative is optical path length
            %  To add a converging lens, use r^2/(2*RoC) with RoC > 0
            obj.setTRfiles( 'ITMX', '', ...
                '-calcThermal(1, r, PsubIX, PcoatIX, wIX, 0.1, 0.175 ) + r.^2/2*invRoCITMXCP', ...
                0.16, 0, 0, 'PsubIX', 0', 'PcoatIX', 0, 'wIX', 0.0353, 'invRoCITMXCP', 0 );
            
            obj.setHRfiles( 'ETMX', './Data/ETM07_CIT.dat', ...
                'map + calcThermal(0, r, 0, PcoatEX, wEX, 0.1, 0.175 ) + r.^2/2*invRoCETMXRH', ...
                0.16, RoCETMX, 0, 'PcoatEX', 0, 'wEX', 0.0353, 'invRoCETMXRH', 0 );

        end
    end
end
