%% using simplest FP
fp0 = FP0;
fp0.lock;
fp0.roundtripLoss
[EI,xI] = fp0.getField('ITM-HR-i', 1.4);
% plot the power distribution
figure(1)
pcolor( xI, xI, log(abs(EI).^2)); shading flat; axis equal; colorbar;

%% setup a FP with default parameter
fp = FPIFO;
% lock
fp.lock
% information about fields and optics
fp.fields.printAll
fp.optics.printAll
% some characteristics
fp.roundtripLoss
fp.gaussFit('ITM-AR-i')
fp.gaussFit('ITM-HR-o')
fp.power('ITM-AR-i')
fp.power('ITM-HR-o')
% mode fractions
HGFrac = fp.HGFrac('ETM-HR-o', 0:5,0:5)
LGFrac = fp.HGFrac('ETM-HR-o', 0:5,0);
% get field at ITM, on HR size, with aperture of 1.4m
[EI,xI] = fp.getField('ITM-HR-i', 1.4);
% plot the power distribution
figure(2)
pcolor( xI, xI, log(abs(EI).^2)); shading flat; axis equal; colorbar;

%% setup a FP with Aidan's map on both ITM and ETM
fpA = FPIFO('aidanMul', 1);
fpA.lock
HGFracA = fpA.HGFrac('ETM-HR-o', 0:5,0:5)
LGFracA = fpA.HGFrac('ETM-HR-o', 0:5,0);
LGFrac
LGFracA
[EE,xE] = fp.getField('ETM-HR-o');

%% setup a FP with 0,1W coating absorption on ITM and ETM
fpH11 = FPIFO('PcoatI', 0.1, 'PcoatE', 0.1);
fpH11.lock
LGFracH11 = fpH11.HGFrac('ETM-HR-o', 0:5,0);
LGFrac

%% plot surface maps
% 3 is ETM
% This is r^2/(2*RoC)
map0 = xE.^2/2*fp.optics.optList(3).invRoC*1e9;
% This is the surface map of the default FP, i.e., mirror phase map + r^2/(2*RoC)
mirrorMap = fp.optics.optList(3).RPYmap/2*1e9;
% Aidan's map + mirrorMap
AidanMap = fpA.optics.optList(3).RPYmap/2*1e9;
% Hello Vinet map + mirrorMap
HelloVinetMap = fpH11.optics.optList(3).RPYmap/2*1e9;
figure(3)
plot( xE, mirrorMap(end/2,:)-map0, xE, AidanMap(end/2,:)-mirrorMap(end/2,:), xE, (HelloVinetMap(end/2,:)-mirrorMap(end/2,:)))
legend('mirror map with coating (=refmap)', 'Aidan 1W deformation - refmap', 'Heating by 0.1W absorption - refmap')
xlabel('radius (m)')
ylabel('height (nm)')
axis([-0.17 0.17 -30 10])
grid

% LLO PRFPM
llo = aLIGOIFO;
llo.lock;
PRG = llo.power('BS-HRx-o')
CD = llo.power('BS-ARy-o') / PRG

