classdef H1PRFPM < FFTIFO
    
    methods
        function obj = H1PRFPM( varargin )
            obj@FFTIFO( varargin{:} );
        end

        function defineIFO( obj, varargin )
            % control parameters
            extraArmLoss = 40e-6;
            rmloss = 1;  % PRC loss
            ITMloss = 1; % ITM AR loss
            angleScale = 1; % RC normal incident
            armScale = 1; % test mass and PRC scaled
            BSScale = 1; % BS scale
            armLoss = 1; % arm HR side loss
            RMSize = -8; % RM size
            TMMap = 1; % use test mass or not
            BSNN = 6;  % which BS
            invFocalY =  0;  % redefine the effective curvature seen from AR side
            invFocalX =  0;
            withArm = 1;  % with or without arm
            dPRC23Corr = 0.00828895;
            PRM3scale = 1;
            
            % Interacive parameter setting
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            % arm optics
            CPXdat = './Data/xCP_01_SPTWE_Figure.dat'; LossCPX = (0.5+10.2+0.3+43.7)*1e-6;
            CPYdat = './Data/xCP_05_SPTWE_Figure.dat'; LossCPY = (0.4+27.2+0.4+5)*1e-6;
            
            dITM = 0.2; dETM = 0.2;
            
            ITMXdat = './Data/ITM03_CIT.dat'; TITMX = 1.39e-2; RoCITMX = 1939.3;
            AbsHRITMX = 0.3e-6; AbsARITMX = 1.1e-6;
            LossHRITMX = 5.6e-6 + AbsHRITMX; LossARITMX = 264e-6 + AbsARITMX + LossCPX;
            ITMYdat = './Data/ITM11_CIT.dat'; TITMY = 1.42e-2; RoCITMY = 1939.2;
            AbsHRITMY = 0.4e-6; AbsARITMY = 1.0e-6;
            LossHRITMY = 12e-6 + AbsHRITMY; LossARITMY = 294e-6 + AbsARITMY + LossCPY;
            
%             TITMX = TITMY;
%             LossHRITMX = LossHRITMY; LossARITMX = LossARITMY;
            
            ITMXTWEdat = './Data/xITM_03_SPTWE_Figure.dat'; RoCITMXTWE = -79.70e3;
            ITMYTWEdat = './Data/xITM_11_SPTWE_Figure.dat'; RoCITMYTWE = 570.0e3;
            
            ETMXdat = './Data/ETM08_CIT.dat'; TETMX = 3.6e-6; RoCETMX = 2241.54;
            AbsETMX = 0.3e-6;
            LossHRETMX = 8e-6 + AbsETMX;
            ETMYdat = './Data/ETM12_CIT.dat'; TETMY = 3.6e-6; RoCETMY = 2238.9;
            AbsETMY = 0.4e-6;
            LossHRETMY = 8.8e-6 + AbsETMY;
            
            % use average of X and Y arm test masses to define modes
            ITMRoC = (RoCITMX + RoCITMY)/2;
            ETMRoC = (RoCETMX + RoCETMY)/2;
            
            % RC optics
            R_PRM = -10.9478; TPRM = 3.1e-2; LossHRPRM = (8 + 0.5)*1e-6; LossARPRM = 35e-6;
            R_PR2 = -4.5443; TPR2 = 229e-6; LossHRPR2 = (10+0.7)*1e-6;
            R_PR3 = 36.021 ; TRP3 = 3.17e-6; LossHRPR3 = (16.5 + 0.45)*1e-6;

            R_SRM = -5.715 ; TSRM = 20.2e-2; LossHRSRM = (7.6 + 0.4)*1e-6; LossARSRM = 63e-6;
            R_SR2 = -6.4057;
            R_SR3 = 35.973;
            
            LossHRBS = (14 + 0.3)*1e-6; LossARBS = (50 + 1.4)*1e-6;
            
            % geometry
            dPRC12 = 16.6128; dPRC23 = 16.1626 + dPRC23Corr; dPRC3B = 19.5381;
            dSRC12 = 15.7400; dSRC23 = 15.4601 -2e-2; dPRC3B = 19.3661;
            
            dCP = 0.10;  dBSCPX = 4.8046;  dBSCPY = 4.8497; dCPITM = 0.02;
            
            thetaPR2 = 0.79/180*pi*angleScale;  thetaPR3 = 0.615/180*pi*angleScale;  
            thetaSR2 = 0.87/180*pi*angleScale;  thetaSR3 = 0.785/180*pi*angleScale;

            % aperture
            Atm = 0.326 * armScale;
            Apr3 = 0.262 * PRM3scale;
            Abs = 0.3671 * BSScale;
            dBS = 0.06;

            % if no arm mas is used, add average arm loss of 35ppm by hand
            if TMMap ~= 0
                armLoss = 1;
                armLossPPM = 0;
            else
                armLoss = 0;
                armLossPPM = 35e-6;
            end
                                            
            % add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', 1/R_PRM, 'Thick', 0.0749, ...
                'T', TPRM, 'Lhr', LossHRPRM*rmloss, 'Lar', LossARPRM*rmloss, 'Aperture', RMSize);
            obj.addMirror( 'PRM2', 'invRoC', 1/R_PR2, ...
                'T', TPR2, 'Lhr', LossHRPR2*rmloss, 'inc', thetaPR2, 'Aperture', RMSize );
            obj.addMirror( 'PRM3', 'invRoC', 1/R_PR3, 'Aperture', Apr3, ...
                'T', TRP3, 'Lhr', LossHRPR3*rmloss, 'incAngle', thetaPR3 );
                        
            obj.addMirror( 'BS', 'Aperture', Abs, 'Thickness', dBS, ...
                'T', 0.5, 'Lhr', LossHRBS, 'Lar', LossHRBS, 'incAngle', pi/4 );
            
            obj.addMirror( 'ITMX', 'invRoC', 1/ITMRoC, 'Aperture', Atm, 'Thick', dITM + dCP, ...
                'T', TITMX, 'Lhr', LossHRITMX*armLoss+armLossPPM/2+extraArmLoss/2, 'Lar', LossARITMX*ITMloss, 'removeTilt', true );
            obj.addMirror( 'ITMY', 'invRoC', 1/ITMRoC, 'Aperture', Atm, 'Thick', dITM + dCP, ...
                'T', TITMY, 'Lhr', LossHRITMY*armLoss+armLossPPM/2+extraArmLoss/2, 'Lar', LossARITMY*ITMloss, 'removeTilt', true );

            obj.addDetector( 'dark' );
 
            % build BS maps
            
            switch BSNN
                case 2
                    obj.setHRfiles( 'BS', './Data/BS02_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.setARfiles( 'BS', './Data/BS02_R2Fig.dat', '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', './Data/BS02_SPTWE.dat', '', 0.16, 0, -1 );
                case 5  % for L1
                    obj.setHRfiles( 'BS', './Data/BS05_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.setARfiles( 'BS', './Data/xBS_05_R2_Figure.dat', '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', './Data/xBS_05_SPTWE_Figure.dat', '', 0.16, 0, -1 );
                case 6  % for H1
                    obj.setHRfiles( 'BS', './Data/BS06_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.setARfiles( 'BS', './Data/xBS_06_R2_Figure.dat', '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', './Data/xBS_06_SPTWE_Figure.dat', '', 0.16, 0, -1 );
                case 100
                    obj.setRPYfiles( 'BS', './Data/BS02_RPY.dat', '', 0.22, 0 );
                    obj.setRSXfiles( 'BS', './Data/BS02_RSX.dat', '', 0.22, 0 );
                    obj.setTPXfiles( 'BS', './Data/BS02_TPX.dat', 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                    obj.setTSYfiles( 'BS', './Data/BS02_TSY.dat', 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                case 101
                    global BSmodel;
                    BSmodel = mphload('BS.mph');
                    obj.setRPYfiles( 'BS', '', '-sqrt(2)*calcBSatZ( ''w'', x(1,end:-1:1), y(:,1), 0, 0 );', 0.22, 0 );
                    obj.setRSXfiles( 'BS', '', '-(calcBST( x(1,:), y(:,1), 0.06, 10, 1 )+calcBST( x(1,:), y(:,1), 0.06, 10, -1 ));', 0.22, 0 );
                    obj.setTPXfiles( 'BS', '', '-calcBST( x(1,:), y(:,1), 0.06, 10, 1 );', 0.22, 0 );
                    obj.setTSYfiles( 'BS', '', '-calcBST( x(1,end:-1:1), y(:,1), 0.06, 10, -1 );', 0.22, 0 );
            end
            
            % implement BS baffle
            if armScale == 1
                obj.setModMap( 'BS', @obj.BSbaffle );
            end
            
            if withArm
                 % ETMX
                 obj.addMirror( 'ETMX', 'invRoC', 1/ETMRoC, 'Aperture', Atm, 'Thick', dETM, 'removeTilt', true, ...
                     'T', TETMX, 'Lhr', LossHRETMX*armLoss+armLossPPM/2+extraArmLoss/2, 'resolution', 2e-3 );
                 % ETMY
                 obj.addMirror( 'ETMY', 'invRoC', 1/ETMRoC, 'Aperture', Atm, 'Thick', dETM, 'removeTilt', true, ...
                     'T', TETMY, 'Lhr', LossHRETMY*armLoss+armLossPPM/2+extraArmLoss/2, 'resolution', 2e-3 );
            end
            
%% test mass maps
           if isempty( TMMap ) || TMMap ~= 0
               % ITMX
               obj.setHRfiles( 'ITMX', ...
                   ITMXdat, ...     % phase map data file
                   'map + (x.^2+y.^2)/2*invRoCRHX', ...    % HR surface RoC change by RH
                   0.16, RoCITMX, 0, 'invRoCRHX', 0);          
               % transmission map + thermal lens by RH and lens by CP
               obj.setTRfiles( 'ITMX', ITMXTWEdat, 'map - (x.^2+y.^2)/2*invRoCRHX*11.1 - (x.^2+y.^2)/2*invFnumCPX', ...
                   0.16, RoCITMXTWE, 0, 'invRoCRHX', 0, 'invFnumCPX', 0);
               obj.setTRfiles( 'ITMX', CPXdat, '', 0.16, 1e10);
               
               % ITMY
               obj.setHRfiles( 'ITMY', ...
                   ITMYdat, ...     % phase map data file
                   'map + (x.^2+y.^2)/2*invRoCRHY', ...
                   0.16, RoCITMY, 0, 'invRoCRHY', 0);                    
                obj.setTRfiles( 'ITMY', ITMYTWEdat, 'map - (x.^2+y.^2)/2*invRoCRHY*11.1 - (x.^2+y.^2)/2*invFnumCPY', ...
                    0.16, RoCITMYTWE, 0, 'invRoCRHY', 0, 'invFnumCPY', 0);
                obj.setTRfiles( 'ITMY', CPYdat, '', 0.16, 1e10);
               
                if withArm
                    obj.setHRfiles( 'ETMX', ETMXdat, 'map + (x.^2+y.^2)/2*invRoCRHETMX', 0.16, RoCETMX, 0, 'invRoCRHETMX', 0 );
                    obj.setHRfiles( 'ETMY', ETMYdat, 'map + (x.^2+y.^2)/2*invRoCRHETMY', 0.16, RoCETMY, 0, 'invRoCRHETMY', 0);
               end
           end
            
%% propagators
            % arms
            if withArm
                 obj.addProp( '', 'ITMX-HR', 'ETMX-HR', 3994.5);
                 obj.addProp( '', 'ITMY-HR', 'ETMY-HR', 3994.5);
            end
            
            obj.addProp( '', 'src', 'PRM-AR' );
            obj.addProp( '', 'PRM-HR', 'PRM2-HRx', dPRC12 );
            obj.addProp( '', 'PRM2-HRy', 'PRM3-HRx', dPRC23 );
            obj.addProp( '', 'PRM3-HRy', 'BS-HRx', dPRC3B );
            
            obj.addProp( '', 'BS-ARx', 'ITMX-AR', dBSCPX + dCPITM);
            obj.addProp( '', 'BS-HRy', 'ITMY-AR', dBSCPY + dCPITM);
            
            % SRC props : uncomment this to add SRC
%             obj.props.addProp( '', 'SRM',  1, 'SRM2', 1, 15.7566 );
%             obj.props.addProp( '', 'SRM2',  2, 'SRM3', 1, 15.4435 );
%             obj.props.addProp( '', 'SRM3',  2, 'BS', 4, 19.368 );
%             obj.props.addProp( '', 'SRM',  2, 'dark', 1 );
% make sure the following one to be commented out when SRC is added
            obj.props.addProp( '', 'BS-ARy', 'dark');
        end
                
        function setupBaseMode( obj, varargin )
            
            useArmMode = 1;
            dZ = 0;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            if useArmMode == 0 || obj.optics.findNamedOptic('ETMX') <= 0
%                 cav = obj.cavities.closedCavs{1};
%                 ind = find( cav(2:end) == cav(1), 1 );
%                 cav = cav(1:ind+1);
%                 [qInv, rootID] = obj.cavities.findCavityMode( cav );
% %              rootID = 1;
% %              qInv = -0.132522 - i*0.0332622;
% %               -1.323621960182562e-01 - 6.849822106634568e-02i
%                qInv = 1.331096021213256e-01*RoCBS - 6.869942690291372e-02i*wBS;
                z = 5.93239; z0 = 3.06178;
                qInv = 1 / ( z - dZ + 1i*z0 );
                [Rinv, w] = MMTK.qinv2RW( qInv )
                rootID = 2;
            else

                % adjust the PRM2-PRM3 length so that the PRC mode matches with the arm mode
                ITMYHRoutID = obj.optics.findPortID( 'ITMY-HR-o' );
                ITMYHRinID = obj.optics.findPortID( 'ITMY-HR-i' );
                ITMYARoutID = obj.optics.findPortID( 'ITMY-AR-o' );
                ETMYHRoutID = obj.optics.findPortID( 'ETMY-HR-o' );
                BSHRxoutID = obj.optics.findPortID( 'BS-HRx-o' );
                PRM3xoutID = obj.optics.findPortID( 'PRM3-HRx-o' );
                PRM2xoutID = obj.optics.findPortID( 'PRM2-HRx-o'  );
                BSARyoutID = obj.optics.findPortID( 'BS-ARy-o' );
     %           SRM3xoutID = obj.optics.findPortID( 'SRM3', 1 );
     %           SRM2xoutID = obj.optics.findPortID( 'SRM2', 1 );

                % resonating mode in the arm.  rootArmY is ITMYHRoutID
                [qInv, rootID] = obj.cavities.findCavityMode( [ITMYHRoutID, ETMYHRoutID] );
                sprintf('Mode used to define the modes of the entire IFO : portID = %d, RoC = %g',...
                    rootID, 1/real(qInv) )

                % find the q value going out of the y arm
                % reverse the q value going into the ITMY HR, then let it goes through ITMY toward AR side
                qInvArmYAR = MMTK.ABCD_qinv( obj.optics.findABCD( ITMYHRinID, ITMYARoutID ), MMTK.reverseDir( qInv ) );

                % find the adjustment of the PRM3-PRM2 length
                LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
                    [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );

                % find the adjustment of the SRM3-SRM2 length
     %           LadjS = obj.cavities.adjustModeMatch( qInvArmYAR, SRM3xoutID, 0, ...
     %               [ITMYARoutID, BSARyoutID, SRM3xoutID, SRM2xoutID ], true );

    %            sprintf('PRM3-PRM2 is adjusted by %g and SRM3-SRM2 by %g', LadjP, LadjS )
                sprintf('PRM3-PRM2 is adjusted by %g', LadjP )
            end
            
            obj.cavities.fillQvals( rootID, qInv );
        end
      end
    
    methods( Static )
        % implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y )
            
            baffleMask = x.^2/0.105^2+y.^2/0.13^2 < 1;
            
            reflOut{1} = baffleMask .* reflIn{1};
            reflOut{2} = reflIn{2} .* fliplr(baffleMask);
            reflOut{3} = reflIn{3} .* fliplr(baffleMask);
            reflOut{4} = baffleMask .* reflIn{4};
            transOut{1} = baffleMask .* transIn{1};
            transOut{2} = transIn{2} .* fliplr(baffleMask);
            transOut{3} = transIn{3} .* fliplr(baffleMask);
            transOut{4} = baffleMask .* transIn{4};
        end
    end
end