%
% mich = MICHIFO;
% 
% mich.fields.printAllInfo
%

classdef IMCIFO < FFTIFO
    
    methods
        
        function obj = IMCIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end

        function defineIFO( obj, varargin )
            RoCMC2 = 27.346;
            L12 = 16.2406;
            L13 = 0.4650;
            thick = 0.0747;
            aperture = 0.15;
            wMC13 = 2.13e-3;
            wMC2 = 3.376e-3;
            T_MC1 = 6150e-6;
            T_MC3 = 6130e-6;
            T_MC2 = 3.5e-6;
            L_MC2 = 12.5e-6;
            theta2 = atan( L13/2/L12 );
            theta1 = (pi/2 - theta2)/2;
            Labs = 1e-6;
            Lsact = 20e-6;
            
            PcoatMC13 = 0*1e-3;
            PcoatMC2 = 0*1e-3;
            withMap = 1;
            
            ind1 = 0;
            ind2 = 0;
            fiekdMode = 'HG';
            
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            obj.optics.addLaser( 'src', 'fiekdMode', fiekdMode, 'ind1', ind1, 'ind2', ind2 );
            
            obj.addMirror( 'MC1', 'Thickness', thick, ...
                'T', T_MC1, 'Lhr', Lsact + Labs, 'Lar', 343e-6, 'incAngle', theta1 );
            obj.addMirror( 'MC3', 'Thickness', thick, ...
                'T', T_MC3, 'Lhr', Lsact + Labs, 'Lar', 305e-6, 'incAngle', theta1 );
            obj.addMirror( 'MC2', 'invRoC', 1/RoCMC2, 'Thickness', 0, ...
                'T', T_MC2, 'Lhr', L_MC2 + Labs, 'Lar', 56e-6, 'incAngle', theta2 );

            obj.optics.addDetector( 'refl' );
            obj.optics.addDetector( 'trans' );
            
            obj.addProp( '', 'src', 'MC1-ARx' );
            obj.addOneProp( 'Prop13', 'MC1-HRx', 'MC3-HRy', L13 );
            obj.addOneProp( 'Prop32', 'MC3-HRx', 'MC2-HRy', L12 );
            obj.addOneProp( 'Prop21', 'MC2-HRx', 'MC1-HRy', L12 );
            obj.addProp( '', 'MC1-ARy', 'refl' );
            obj.addProp( '', 'MC3-ARy', 'trans' );
            
            % calculate modes
%            obj.analyzeCavity;
%            obj.setupBaseMode( varargin{:} );
%            wMC13 = obj.fields.fieldList( obj.fields.findNamedField( 'MC1-HRx-o', -1 ) ).w;
%            wMC2 = obj.fields.fieldList( obj.fields.findNamedField( 'MC2-HRx-o', -1 ) ).w;

%           if withMap
               obj.setHRfiles('MC1', './Data/IMCF-02-A-S1.dat', ...
                   'withMap*map + calcThermal(0, r, 0, PcoatMC13, wMC13, thick, aperture )', ...
                   0.04, 1e20, 0, 'PcoatMC13', PcoatMC13, 'wMC13', wMC13, 'thick', thick, 'aperture', aperture, 'withMap', withMap );
               obj.setHRfiles('MC3', './Data/IMCF-03-A-S1.dat', ...
                   'withMap*map + calcThermal(0, r, 0, PcoatMC13, wMC13, thick, aperture )', ...
                   0.04, 1e20, 0, 'PcoatMC13', PcoatMC13, 'wMC13', wMC13, 'thick', thick, 'aperture', aperture, 'withMap', withMap );
               obj.setHRfiles('MC2', './Data/IMCC03-S1.dat', ...
                   'withMap*map + calcThermal(0, r, 0, PcoatMC2, wMC2, thick, aperture )', ...
                   0.02, RoCMC2, 0, 'PcoatMC2', PcoatMC2, 'wMC2', wMC2, 'thick', thick, 'aperture', aperture, 'withMap', withMap );
%           end
        end
        
%         function setupBaseMode( obj, qInv )
%             if ~exist( 'qInv', 'var' )
%                 qInv = -0.000746852 - 1i*0.000119701;
%             end
%             root = obj.optics.findPortID( 'src', 1 );
%             obj.cavities.fillQvals( root, qInv );
%         end
    end
end