%
% LLOPRMI = aLIGOIFO;
%
% building IFO
%   defineIFO
%       addLaser( name ); addDetector( name );
%       addProp( name, fromOpt, fromPort, toOpt, toPort, L0, delL )
%       addMap( fileName, formula, ApertureFit, RoCFit, rotMap, varargin )
%
%   setupBaseMode
%       fillQvals( root, invQ )
%
% analysis tools
%   ifo = LLOPRMI;
%   ifo.lock;
%
%   [E, x] = ifo.getField('ITMy-AR-i');
%       mesh( x, x, log(abs(E).^2 )
%   ifo.gaussFit('ITMy-AR-i')
%   ifo.HGCoef('ITMy-AR-i', 0:3,0:3)
%   ifo.optics.optList(ITMXID).updateMirMaps('invRoC', iRoC*1e-6);
%   ifo.optics.refl( fieldID );
%   ifo.optics.trans( fieldID );
%

classdef LLOPRMI < FFTIFO
    
    methods
        function defineIFO( obj )
            
            dPRC12 = 16.6107; dPRC23 = 16.1647; dPRC3B = 19.5384;
            dCP = 0.10;  dBSCPX = 4.8046;  dBSCPY = 4.8497; dCPITM = 0.005;
            thetaPR2 = 0.79/180*pi;  thetaPR3 = 0.615/180*pi;  
            thetaSR2 = 0.87/180*pi;  thetaSR3 = 0.785/180*pi;
            
% add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', -1/11.009, 'Thick', 0.0749, ...
                'T', 3.1e-2, 'Lhr', (15+0.5)*1e-6, 'Lar', (11.45+26)*1e-6 );
            obj.addMirror( 'PRM2', 'invRoC', -1/4.545, ...
                'T', 243e-6, 'Lhr', 8.6e-6, 'inc', thetaPR2 );
            obj.addMirror( 'PRM3', 'invRoC', 1/36.027, 'Aperture', 0.256, ...
                'T', 5.3e-6, 'Lhr', 17e-6, 'incAngle', thetaPR3 );
            
            % SRC mirrors : uncomment this section to add SRC mirrors
            % SRM size is set to be 10 times of beam size
             obj.addMirror( 'SRM', 'invRoC', -1/5.677,  'Aperture', -12,  'Thick', 0.075, ...
                 'T', 20.6e-2, 'Lhr', 8.7e-6, 'Lar', 50e-6 );
             obj.addMirror( 'SRM2', 'invRoC', -1/6.406, 'T', 18.3e-6, 'incAngle', thetaSR2 );
             obj.addMirror( 'SRM3', 'invRoC', 1/35.97,  'Aperture', 0.256, 'T', 5e-6, 'inc', thetaSR3 );

            BSID = obj.addMirror( 'BS', 'Aperture', 0.366, 'Thickness', 0.06, ...
                'T', 0.5, 'Lhr', 8.6e-6, 'Lar', 31.7e-6, 'incAngle', pi/4 );
            ITMXID = obj.addMirror( 'ITMX', 'invRoC', 1/1934, 'Aperture', 0.34, 'Thick', 0.2 + dCP, ...
                'T', 1.48e-2, 'Lhr', 10.4e-6, 'Lar', (164+42)*1e-6, 'removeTilt', true, 'resolution', 3e-3 );
            ITMYID = obj.addMirror( 'ITMY', 'invRoC', 1/1934, 'Aperture', 0.34, 'Thick', 0.2 + dCP, ...
                'T', 1.48e-2, 'Lhr', 14.3e-6, 'Lar', (250+80)*1e-6, 'removeTilt', true );
            
            obj.addDetector( 'dark' );
            
            % build BS maps
            obj.optics.optList(BSID).HRfiles.addMap( './Data/BS02_S1_CIT.dat', '', 0.16, 0, 1 );
            obj.optics.optList(BSID).ARfiles.addMap( './Data/BS02_R2Fig.dat', '', 0.16, 0, -1 );
            obj.optics.optList(BSID).TRfiles.addMap( './Data/BS02_SPTWE.dat', '', 0.16, 0, -1 );

%              obj.optics.optList(BSID).RPYfiles.addMap( './Data/BS02_RPY.dat', '', 0.22, 0 );
%              obj.optics.optList(BSID).RSXfiles.addMap( './Data/BS02_RSX.dat', '', 0.22, 0 );
%              obj.optics.optList(BSID).TPXfiles.addMap( './Data/BS02_TPX.dat', '', 0.22, 0 );
%              obj.optics.optList(BSID).TSYfiles.addMap( './Data/BS02_TSY.dat', '', 0.22, 0 );

            % implement BS baffle
            obj.optics.optList(BSID).modMap = @obj.BSbaffle;
            
            % ITMX
             obj.optics.optList(ITMXID).HRfiles.addMap( ...
                 './Data/ITM04_CITZygo.dat', ...     % phase map data file
                 'map + (x.^2+y.^2)/2*invRoC', ...   % formula to modify the map : change the RoC for simple TCS
                 0.16, 1937.9, 0, ...                % BS RoC, 1937.9m, was measured in the central 16cm 
                 'invRoC', 0.95/100.46e3);              % default value of the parameter in the formula
             obj.optics.optList(ITMXID).TRfiles.addMap( './Data/itm04_sptwe.dat', '', 0.16, 302.41e3);
%             obj.optics.optList(ITMXID).TRfiles.addMap( '', ' -(x.^2+y.^2)/2/302.41e3', 0.16, 1e10);
              obj.optics.optList(ITMXID).TRfiles.addMap( './Data/CP06_sptwe.dat', '', 0.16, 0);

            % ITMY
             obj.optics.optList(ITMYID).HRfiles.addMap( './Data/ITM08_CITZygo.dat', '', 0.16, 1940.7);
%             obj.optics.optList(ITMYID).TRfiles.addMap( '', '(x.^2+y.^2)/2/82.00e3', 0.16, 1e10);
             obj.optics.optList(ITMYID).TRfiles.addMap( './Data/itm08_sptwe.dat', '', 0.16, -82.00e3);
%             obj.optics.optList(ITMYID).TRfiles.addMap( './Data/itm08_sptwe.dat', 'ind=find(x.^2+y.^2>0.08^2); map(ind)=0;func=map;', 0.16, 82.00e3);
              obj.optics.optList(ITMYID).TRfiles.addMap( './Data/CP08_sptwe.dat', '', 0.16, 0);

%{
            % uncommnt this section to add arms
            % ETMX
            ETMXID = obj.addMirror( 'ETMX', 1/2245, 0.34, 0.2, 3.7e-6, armLoss/2 );
            obj.optics.optList(ETMXID).HRfiles.addMap( './Data/ETM07_CIT.dat', '', 0.16, 2239.7);
            % set the resolution on ETM
   	        obj.optics.optList(ETMXID).resolution = 0.002;
            
            % ETMY
            ETMYID = obj.addMirror( 'ETMY', 1/2245, 0.34, 0.2, 3.7e-6, armLoss/2 );
            obj.optics.optList(ETMYID).HRfiles.addMap( './Data/ETM09_CIT.dat', '', 0.16, 2242.4);
            % set the resolution on ETM
   	        obj.optics.optList(ETMYID).resolution = 0.002;
            
            % arms
            obj.props.addProp( '', 'ITMX',  1, 'ETMX', 1, 3994.5);
            obj.props.addProp( '', 'ITMY',  1, 'ETMY', 1, 3994.5);
%}
            obj.props.addProp( '', 'src', 1, 'PRM', 2 );
            obj.props.addProp( '', 'PRM',  1, 'PRM2', 1, dPRC12 );
            obj.props.addProp( '', 'PRM2',  2, 'PRM3', 1, dPRC23 );
            obj.props.addProp( '', 'PRM3',  2, 'BS', 1, dPRC3B );
            
            obj.props.addProp( '', 'BS',  3, 'ITMX', 2, dBSCPX + dCPITM);
            obj.props.addProp( '', 'BS',  2, 'ITMY', 2, dBSCPY + dCPITM);
            
            % SRC props : uncomment this to add SRC
%             obj.props.addProp( '', 'SRM',  1, 'SRM2', 1, 15.7566 );
             obj.props.addOneProp( '', 'BS',  4, 'SRM3', 1, 19.368 );
             obj.props.addOneProp( '', 'SRM3',  2, 'SRM2', 1, 15.4435 );
             obj.props.addOneProp( '', 'SRM2',  2, 'SRM', 1, 15.7566 );
             obj.props.addOneProp( '', 'SRM',  2, 'dark', 1, 3.5 );

             %             obj.props.addProp( '', 'SRM',  2, 'dark', 1 );
% make sure the following one to be commented out when SRC is added
%            obj.props.addProp( '', 'BS',  4, 'dark', 1 );
        end
                
        % simple mode injector
         function setupBaseMode( obj )
             % this was manuallly calculated to match with the code PRC-BS-Y arm AR
%              qInv =  0.132039 - 1i * 0.0359203;
%              root = obj.optics.findPortID( 'src', 1 );
             qInv = 9.083477155054981e-02 - 3.380560739848366e-02i;
             root = 5;
             % inject the mode using the root field and propagate to all fields
             obj.cavities.fillQvals( root, qInv );
         end
        
        % This uses a cavity to define a mode
%         function setupBaseMode( obj )
%             [qInv, root] = obj.cavities.findCavityMode( obj.cavities.closedCavs{1} );            
%             obj.cavities.fillQvals( root, qInv );
%         end
%{
% this adjusts the PRM2-PRM3 length to make the PRC mode matches with the arm mode
        function setupBaseMode( obj )
            % adjust the PRM2-PRM3 length so that the PRC mode matches with the arm mode
            ITMYHRoutID = obj.optics.findPortID( 'ITMY', 1 );
            ITMYHRinID = obj.optics.findPortID( 'ITMY', -1 );
            ETMYHRoutID = obj.optics.findPortID( 'ETMY', 1 );
            ITMYARoutID = obj.optics.findPortID( 'ITMY', 2 );
            BSHRxoutID = obj.optics.findPortID( 'BS', 1 );
            PRM3xoutID = obj.optics.findPortID( 'PRM3', 1 );
            PRM2xoutID = obj.optics.findPortID( 'PRM2', 1 );
            BSARyoutID = obj.optics.findPortID( 'BS', 4 );
 %           SRM3xoutID = obj.optics.findPortID( 'SRM3', 1 );
 %           SRM2xoutID = obj.optics.findPortID( 'SRM2', 1 );
            
            % resonating mode in the arm.  rootArmY is ITMYHRoutID
            [qInvArmY, rootArmY] = obj.cavities.findCavityMode( [ITMYHRoutID, ETMYHRoutID] );
            sprintf('Mode used to define the modes of the entire IFO : portID = %d, RoC = %g',...
                rootArmY, 1/real(qInvArmY) )
            
            % find the q value going out of the y arm
            % reverse the q value going into the ITMY HR, then let it goes through ITMY toward AR side
            qInvArmYAR = MMTK.ABCD_qinv( obj.optics.findABCD( ITMYHRinID, ITMYARoutID ), MMTK.reverseDir( qInvArmY ) );
            
            % find the adjustment of the PRM3-PRM2 length
            LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
                [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
            
            % find the adjustment of the SRM3-SRM2 length
 %           LadjS = obj.cavities.adjustModeMatch( qInvArmYAR, SRM3xoutID, 0, ...
 %               [ITMYARoutID, BSARyoutID, SRM3xoutID, SRM2xoutID ], true );
            
%            sprintf('PRM3-PRM2 is adjusted by %g and SRM3-SRM2 by %g', LadjP, LadjS )
            
            obj.cavities.fillQvals( rootArmY, qInvArmY );
        end
        %}

      end
    
    methods( Static )
        % implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y )
            
            baffleMask = x.^2/0.105^2+y.^2/0.13^2 < 1;
            
            reflOut{1} = baffleMask .* reflIn{1};
            reflOut{2} = reflIn{2} .* fliplr(baffleMask);
            reflOut{3} = reflIn{3} .* fliplr(baffleMask);
            reflOut{4} = baffleMask .* reflIn{4};
            transOut{1} = baffleMask .* transIn{1};
            transOut{2} = transIn{2} .* fliplr(baffleMask);
            transOut{3} = transIn{3} .* fliplr(baffleMask);
            transOut{4} = baffleMask .* transIn{4};
        end
    end
end