%% object defining the setup
classdef MirrorRefFar < FFTIFO
    
    methods
        %% to support runtime variables
        function obj = MirrorRefFar( varargin )
            obj@FFTIFO( varargin{:} );
        end

        %% definition of IFO
        function defineIFO( obj, varargin )
            
            %% variables, which can be changed interactively by UtilTK.setVars
            incAng = 0;     % incident angle to the mirror
            Ldet = 143;     % distance between the mirror and detector
            RoCMC2 = 185.1; % RoC of the mirror
            res = 0.001;    % resolution of the miror map
            
            %% this is to change variables defined above by using runtime argument, 
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            %% setup optics
            obj.addLaser( 'src' );
            obj.addMirror( 'MC2', 'invRoC', 1/RoCMC2, 'Thickness', 0.0, 'T', 0, 'incAngle', incAng, 'resolution', res );
            obj.addDetector( 'reflMon' );
            
            %% connect optics   : help FFTIFO.addProp
            obj.addOneProp( '', 'src-out', 'MC2-HRx' );
            obj.addProp( '', 'MC2-HRy', 'reflMon-1', Ldet );  % detector name is somehow 'name-1', bad legacy to be fixed
            
            %% define the HR surface  : help FFTIFO.setHRfiles, FFTIFT.setTRfiles, FFTIFO.setARfiles
%            obj.setHRfiles( 'MC2', 'MC_end_mirror_map_surface_2.col', '', 0.130/2, 185.1 );

        end
        
        %% define the input beam mode
        function setupBaseMode( obj, varargin )
            RoCMC2 = 185.1;
            w = 0.0108;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            % invq : 1/R - i lambda/pi 1/w^2 => -1/R - i lambda/pi 1/w^2
            qInv = 1/RoCMC2 - 1i*1.064e-6/pi/w^2;
            % field ID of laser source
            root = obj.optics.findLaser;
            % mode is defined by specifying the field ID and the q value
            obj.cavities.fillQvals( root, qInv );
        end
    end
end