%
% Relection by a Mirror
%
classdef MirrorRefIFO < FFTIFO

    methods       
        function obj = MirrorRefIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end
        
        function defineIFO( obj, varargin )
            Aperture = 0.326;
            fieldMode = 'LG';
            ind1 = 0;
            ind2 = 0;
            TMdat = { ...
                './data/ITM04_CITZygo.dat', './data/ITM08_CITZygo.dat', ...  % LLO ITMs
                './data/ITM03_CIT.dat', './data/ITM11_CIT.dat', ... % LHO ITMs
                './data/ETM07_CIT.dat', './data/ETM09_CIT.dat', ... % LLO ETMs
                './data/ETM08_CIT.dat', './data/ETM12_CIT.dat', ... % LHO ETMs
                '' ... % empty
                }; 
            TMid = 5;
            Pcoat = 0; % 800e3 * 0.3e-6;
            RoC = 1e20;
            w = 1e20;
            res = 2e-3;
            
            WfftUser = 0;
            
            eval( UtilTK.setVars( who, false, varargin{:} ) );
  
            if ~( TMid >= 1 && TMid <= 9 )
                TMid = 9;
            end
            
            if TMid <= 4
                % ITM
                RoC = 1934;
                w = 0.053;
            elseif TMid <= 8
                % ETM
                RoC = 2245;
                w = 0.062;
            end
            
             % setup optics
            obj.optics.addLaser( 'src', 'power', 1, 'fieldMode', fieldMode, 'ind1', ind1, 'ind2', ind2 );
            obj.addMirror( 'TM', 'Thickness', 0.0, 'T', 0, 'resolution', res, 'invRoC', 1/RoC, ...
                'Aperture', Aperture, 'WfftUser', WfftUser );
            obj.addDetector( 'reflMon' );

            obj.addOneProp( '', 'src', 'TM-HR', 0 );
            obj.addProp( '', 'TM-HR', 'reflMon', 0 );
 
            obj.setHRfiles( 'TM', TMdat{TMid}, 'map + calcThermal(0, r, 0, Pcoat, w, 0.2, 0.17 ) + r.^2/2*invRoCRH', ...
                0.16, RoC, 0, 'Pcoat', Pcoat, 'w', w, 'invRoCRH', 0 );
        end
        
        % invq : 1/R - i lambda/pi 1/w^2 => -1/R - i lambda/pi 1/w^2
        function setupBaseMode( obj, varargin )
            TMid = 5;
            RoC = 1e20;
            w = 1e20;
            eval( UtilTK.setVars( who, false, varargin{:} ) );
            if ~( TMid >= 1 && TMid <= 9 )
                TMid = 9;
            end

            if TMid <= 4
                RoC = 1934;
                w = 0.053;
            elseif TMid <= 8
                RoC = 2245;
                w = 0.062;
            end
            
            qInv = 1/RoC - 1i*1.064e-6/pi/w^2;
            root = obj.optics.findPortID( 'src', 1 );
            obj.cavities.fillQvals( root, qInv );
        end
    end
end
