function dat = RanaAnal( lamCR, location, maxData )
% dat = RanaAnal( lamCR, location, maxData )
% use ITM and ETM Zygo polished map data located in 'location', and
% calculate power loss and 00 mode loss for the wavelength of lamCR
% maxData is the maximal number of data used, mostly for test run. Set it
% to 0 to use all data

if ~exist('lamCR', 'var')
    lamCR = 1.064e-6;
end
if ~exist('location', 'var')
    location = './data/';
end
if ~exist('maxData', 'var')
    maxData = 4;
end

% find ITM and ETM data files and combine
[TMnames,TMIDs] = getZygoFiles( 'ITM', location);
[ETMnames,ETMIDs] = getZygoFiles( 'ETM', location);

TMnames(length(TMnames)+1:length(TMnames)+length(ETMnames)) = ETMnames;
TMIDs(length(TMIDs)+1:length(TMIDs)+length(ETMIDs)) = -ETMIDs;

if maxData > 0
    numData = min( length(TMnames), maxData );
end

dat = repmat( struct( 'ID1', 0, 'ID2', 0, 'Lpower', 0, 'Lmode', 0 ), 1, ceil((numData/2+1)*numData/2/2) );
counter = 0;
for i1 = 1 : 2: numData
    for i2 = i1+1 : 2 : numData
        counter = counter + 1;
        dat(counter).ID1 = TMIDs(i1);
        dat(counter).ID2 = TMIDs(i2);
        ITMmap = strcat(location,TMnames{i1});
        ETMmap = strcat(location,TMnames{i2});
        fp = RanaFP( 'lamCR', lamCR', 'ITMmap', ITMmap, 'ETMmap', ETMmap );
        fp.lock;
        dat(counter).Lpower = fp.roundtripLoss;
        dat(counter).Lmode = 1 - fp.LGFrac('ETM-HR-o', 0, 0);
    end
end
