classdef aLIGOIFO < FFTIFO
    %% classdef aLIGOIFO < FFTIFO
    % ------------------------------------------------------
    % aLIGO IFO with or without arms / SRC
    % ------------------------------------------------------
    % parameters  'NAME' val
    %    'IFO'     :  'L' for LLO,     'H' for LHO
    %    'withArm' :   1  for with arm, 0  for without arm
    %    'withSRC' :   1  for with SRC, 0  for without SRC
    % ------------------------------------------------------

    methods
        function obj = aLIGOIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end
        
        function defineIFO( obj, varargin )
            
            IFO = 'L';
            withArm = 1;    % = 1 with arm, = 0 witharm arm
            withSRC = 0;
            useCoated = 1;
            IFOconf = '';
            
            extraArmLossX = 40e-6;  % extra loss per arm to make the arm loss around 80-90ppm
            extraArmLossY = 40e-6;  % extra loss per arm to make the arm loss around 80-90ppm

            tiltETMX = 0;   % ETMX tilt angle
            tiltETMY = 0;   % ETMY tilt angle
            % multiplied to parameters
            rmloss = 1;     % RM cavity loss
            ITMloss = 1;    % ITM loss
            angleScale = 1; % RM cavity incident angle
            armScale = 1;   % test mass aperture
            BSScale = 1;
            useBSBaffle = 1;
            dxBSBaf = 0;
            RMSize = -8;    % RM radius = beam size * RMSize
            TMMap = 1;      % = 1 : use actual mirror data, = 0 : no mirror data
            invFocalY =  1/50e3;  % facal length used to calculate the RoCs with small thermal lensing
            invFocalX =  1/50e3;  % this changes the mode base calculation, but not the actual resonating mode
            ETMXY_Resolution = 0.002; % ETM mass resolution
            
            RoC0_ITM = 1940; % nominal ITM RoC, used for mode definition
            RoC0_ETM = 2240; % nominal ETM RoC, used for mode definition
            dTestMass = 0.2;
            
            BSmapScale = 1;
            
            Abs = 0.37;  % BS aperture, default
            dBS = 0.06;  % BS thickness, default

            fprintf(1, 'first setVals\n');
            eval( UtilTK.setVars( who, true, varargin{:} ) );
              
            Atm = 0.326 * armScale;  % test mass aperture
            Apr3 = 0.262 * armScale; % PRM3 aperture

            if IFO == 'L'
                L1setup;
            else
                H1setup;
            end
                        
            if TMMap ~= 0
                armLoss = 1;
                armLossPPM = 0;
            else
                armLoss = 0;
                armLossPPM = 35e-6;
            end
            
            % Interacive parameter setting
            fprintf(1, 'second setVals\n');
            eval( UtilTK.setVars( who, true, varargin{:} ) );
                        
            if strcmpi(IFOconf, 'PRMI')
                withArm = 0;
                withSRC = 0;
            elseif strcmpi(IFOconf, 'DRMI')
                withArm = 0;
                withSRC = 1;
            elseif strcmpi(IFOconf, 'PRFPM')
                withArm = 1;
                withSRC = 0;
            elseif strcmpi(IFOconf, 'SRFPM')
                withArm = 1;
                withSRC = 1;
            end
            
            % add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', 1/PRMRoC, 'Thick', dPRM, ...
                'T', T_PRM, 'Lhr', HRLoss_PRM*rmloss, 'Lar', ARLoss_PRM*rmloss, 'Aperture', RMSize);
            obj.addMirror( 'PRM2', 'invRoC', 1/PR2RoC, ...
                'T', T_PR2, 'Lhr', HRLoss_PR2*rmloss, 'inc', thetaPR2*angleScale, 'Aperture', RMSize );
            obj.addMirror( 'PRM3', 'invRoC', 1/PR3RoC, 'Aperture', Apr3, ...
                'T', T_PR3, 'Lhr', HRLoss_PR3*rmloss, 'incAngle', thetaPR3*angleScale );
            
            if withSRC
                obj.addMirror( 'SRM', 'invRoC', 1/SRMRoC, 'Thick', dSRM, ...
                    'T', T_SRM, 'Lhr', HRLoss_SRM*rmloss, 'Lar', ARLoss_SRM*rmloss, 'Aperture', RMSize);
                obj.addMirror( 'SRM2', 'invRoC', 1/SR2RoC, ...
                    'T', T_SR2, 'Lhr', HRLoss_SR2*rmloss, 'inc', thetaSR2*angleScale, 'Aperture', RMSize );
                obj.addMirror( 'SRM3', 'invRoC', 1/SR3RoC, 'Aperture', Apr3, ...
                    'T', T_SR3, 'Lhr', HRLoss_SR3*rmloss, 'incAngle', thetaSR3*angleScale );
            end
            
            BSID = obj.addMirror( 'BS', 'Aperture', Abs, 'Thickness', dBS, ...
                'T', T_BS, 'Lhr', HRLoss_BS, 'Lar', ARLoss_BS, 'incAngle', pi/4 );
            ITMXID = obj.addMirror( 'ITMX', 'invRoC', 1/RoC0_ITM, 'Aperture', Atm, 'Thick', dTestMass + dCP, ...
                'T', T_ITMX, 'Lhr', HRLoss_ITMX*armLoss+armLossPPM/2+extraArmLossX/2, 'Lar', ARLoss_ITMX*ITMloss, 'removeTilt', true, ...
                'invFocal', invFocalX);
            obj.addMirror( 'ITMY', 'invRoC', 1/RoC0_ITM, 'Aperture', Atm, 'Thick', dTestMass + dCP, ...
                'T', T_ITMY, 'Lhr', HRLoss_ITMY*armLoss+armLossPPM/2+extraArmLossY/2, 'Lar', ARLoss_ITMY*ITMloss, 'removeTilt', true, ...
                'invFocal', invFocalY);
            
            obj.addDetector( 'dark' );
            
            % build BS maps
            BSsetup;
            
            % implement BS baffle
            if useBSBaffle ~= 0
                var = sprintf('dx = %g;', dxBSBaf);
                obj.setModMap( 'BS', @obj.BSbaffle, var );
            end
            
            if withArm
                % ETMX
                obj.addMirror( 'ETMX', 'invRoC', 1/RoC0_ETM, 'Aperture', Atm, 'Thick', dTestMass, 'removeTilt', tiltETMX == 0, ...
                    'T', T_ETMX, 'Lhr', HRLoss_ETMX*armLoss+armLossPPM/2+extraArmLossX/2, 'resolution', ETMXY_Resolution );
                % ETMY
                obj.addMirror( 'ETMY', 'invRoC', 1/RoC0_ETM, 'Aperture', Atm, 'Thick', dTestMass, 'removeTilt', tiltETMY == 0, ...
                    'T', T_ETMY, 'Lhr', HRLoss_ETMY*armLoss+armLossPPM/2+extraArmLossY/2, 'resolution', ETMXY_Resolution );
            end
            
            %% test mass maps
            if isempty( TMMap ) || TMMap ~= 0
                % ITMX
                if ~isempty( ITMXmap )
                    obj.setHRfiles( 'ITMX', ...
                        ITMXmap, ...     % phase map data file
                        'map - (x.^2+y.^2)/2*RHinvRoCITMX', ...   % formula to modify the map : change the RoC for simple TCS
                        0.16, ITMXRoC, 0, ...                 % BS RoC, 1937.9m, was measured in the central 16cm
                        'RHinvRoCITMX', 0);                         % default value of the parameter in the formula
                end
                if ~isempty( ITMXSPTWE )
                    obj.setTRfiles( 'ITMX', ITMXSPTWE, ...
                        'map + (x.^2+y.^2)/2*CPinvFnumX', 0.16, ITMXSPTWE_RoC, 0, ... % CP thermal lens
                        'CPinvFnumX', CPinvFnumX );
                end
                if ~isempty( CPXmap )
                    obj.setTRfiles( 'ITMX', CPXmap, '', 0.16, 1e10);
                end
                
                % ITMY
                if ~isempty( ITMYmap )
                    obj.setHRfiles( 'ITMY', ...
                        ITMYmap, ...     % phase map data file
                        'map - (x.^2+y.^2)/2*RHinvRoCITMY', ...   % formula to modify the map : change the RoC for simple TCS
                        0.16, ITMYRoC, 0, ...                 % BS RoC, 1937.9m, was measured in the central 16cm
                        'RHinvRoCITMY', 0);                         % default value of the parameter in the formula
                end
                if ~isempty( ITMYSPTWE )
                    obj.setTRfiles( 'ITMY', ITMYSPTWE, ...
                        'map + (x.^2+y.^2)/2*CPinvFnumY', 0.16, ITMYSPTWE_RoC, 0, ... % CP thermal lens
                        'CPinvFnumY', CPinvFnumY );
                end
                if ~isempty( CPYmap )
                    obj.setTRfiles( 'ITMY', CPYmap, '', 0.16, 1e10);
                end
                
                if withArm
                    if ~isempty( ETMXmap )
                        obj.setHRfiles( 'ETMX', ETMXmap, 'map - (x.^2+y.^2)/2*RHinvRoCETMX + x*tiltETMX', ...
                            0.16, ETMXRoC, 0, 'RHinvRoCETMX', 0, 'tiltETMX', tiltETMX);
                    end
                    if ~isempty( ETMYmap )
                        obj.setHRfiles( 'ETMY', ETMYmap, 'map - (x.^2+y.^2)/2*RHinvRoCETMY + x*tiltETMY', ...
                            0.16, ETMYRoC, 0, 'RHinvRoCETMY', 0, 'tiltETMY', tiltETMY);
                    end
                end
            end
            
            %% propagators
            % arms
            if withArm
                obj.addProp( '', 'ITMX-HR', 'ETMX-HR', 3994.5);
                obj.addProp( '', 'ITMY-HR', 'ETMY-HR', 3994.5);
            end
            
            obj.addProp( '', 'src', 'PRM-AR' );
            obj.addProp( '', 'PRM-HR', 'PRM2-HRx', dPRC12 );
            obj.addProp( '', 'PRM2-HRy', 'PRM3-HRx', dPRC23 );
            obj.addProp( '', 'PRM3-HRy', 'BS-HRx', dPRC3B );
            
            obj.addProp( '', 'BS-ARx', 'ITMX-AR', dBSCPX + dCPITM);
            obj.addProp( '', 'BS-HRy', 'ITMY-AR', dBSCPY + dCPITM);
            
%             LRM2BS = dPRC12 + dPRC23 + dPRC3B;
%             LBS2ITMX = dBSCPX + dCPITM + obj.optics.optList(BSID).thicknessInSubs(1) + (dTestMass + dCP)*obj.optics.optList(ITMXID).nRef;
%             LBS2ITMY = dBSCPY + dCPITM + (dTestMass + dCP)*obj.optics.optList(ITMXID).nRef;
%             MichLeng = LRM2BS + (LBS2ITMX + LBS2ITMY)/2;
%             SchAym = LBS2ITMX - LBS2ITMY;
%             fprintf(1, 'MichLeng = %14.10g, Schnupp Asym = %14.10g, RF = %14.10g\n', MichLeng, SchAym, ConstantTK.c / MichLeng*1.75 );
%             fprintf(1, 'dRM-BS = %14.10g, , BSHR-XARM = %14.10g, , BSHR-YARM = %14.10g\n', LRM2BS, LBS2ITMX, LBS2ITMY );
            
            if withSRC
                obj.addProp( '', 'SRM-HR', 'SRM2-HRx', dSRC12 );
                obj.addProp( '', 'SRM2-HRy', 'SRM3-HRx', dSRC23 );
                obj.addProp( '', 'SRM3-HRy', 'BS-ARy', dSRC3B );
                obj.addProp( '', 'SRM-AR', 'dark' );
            else
                obj.addProp( '', 'BS-ARy', 'dark');
            end
        end
        
        function setupBaseMode( obj, varargin )
            
            useArmMode = 1;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            useSRC = obj.optics.findNamedOptic('SRM') > 0;
            
            if useArmMode == 0 || obj.optics.findNamedOptic('ETMX') <= 0
%                 cav = obj.cavities.closedCavs{1};
%                 ind = find( cav(2:end) == cav(1), 1 );
%                 cav = cav(1:ind+1);
%                 [qInv, rootID] = obj.cavities.findCavityMode( cav );
                  rootID = 1;
                  qInv = -1.318657072498640e-01 - 6.818783109537355e-02i;
            else
                LadjS = nan;
                
                % adjust the PRM2-PRM3 length so that the PRC mode matches with the arm mode
                ITMYHRoutID = obj.optics.findPortID( 'ITMY-HR-o' );
                ITMYHRinID = obj.optics.findPortID( 'ITMY-HR-i' );
                ITMYARoutID = obj.optics.findPortID( 'ITMY-AR-o' );
                ETMYHRoutID = obj.optics.findPortID( 'ETMY-HR-o' );
                BSHRxoutID = obj.optics.findPortID( 'BS-HRx-o' );
                PRM3xoutID = obj.optics.findPortID( 'PRM3-HRx-o' );
                PRM2xoutID = obj.optics.findPortID( 'PRM2-HRx-o'  );
                BSARyoutID = obj.optics.findPortID( 'BS-ARy-o' );
                if useSRC
                    SRM3xoutID = obj.optics.findPortID( 'SRM3-HRx-o' );
                    SRM2xoutID = obj.optics.findPortID( 'SRM2-HRx-o' );
                end
                
                % resonating mode in the arm.  rootArmY is ITMYHRoutID
                [qInv, rootID] = obj.cavities.findCavityMode( [ITMYHRoutID, ETMYHRoutID] );
                sprintf('Mode used to define the modes of the entire IFO : portID = %d, RoC = %g',...
                    rootID, 1/real(qInv) )
                
                % find the q value going out of the y arm
                % reverse the q value going into the ITMY HR, then let it goes through ITMY toward AR side
                qInvArmYAR = MMTK.ABCD_qinv( obj.optics.findABCD( ITMYHRinID, ITMYARoutID ), MMTK.reverseDir( qInv ) );
                
                % find the adjustment of the PRM3-PRM2 length
                LadjP = obj.cavities.adjustModeMatch( qInvArmYAR, PRM3xoutID, 0, ...
                    [ITMYARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
                
                % find the adjustment of the SRM3-SRM2 length
                fprintf(1, 'PRM3-PRM2 is adjusted by %g', LadjP );

                if useSRC
                    LadjS = obj.cavities.adjustModeMatch( qInvArmYAR, SRM3xoutID, 0, ...
                        [ITMYARoutID, BSARyoutID, SRM3xoutID, SRM2xoutID ], true );
                    fprintf( 1, ' and SRM3-SRM2 by %g', LadjS );
                else
                    fprintf( 1, '\n');
                end
            end
            
            obj.cavities.fillQvals( rootID, qInv );
        end
    end
    
    methods( Static )
        % implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y, var )
            
            dx = 0;
            eval( var );
            baffleMask = (x-dx).^2/0.105^2+y.^2/0.13^2 < 1;
            
            reflOut{1} = baffleMask .* reflIn{1};
            reflOut{2} = reflIn{2} .* fliplr(baffleMask);
            reflOut{3} = reflIn{3} .* fliplr(baffleMask);
            reflOut{4} = baffleMask .* reflIn{4};
            transOut{1} = baffleMask .* transIn{1};
            transOut{2} = transIn{2} .* fliplr(baffleMask);
            transOut{3} = transIn{3} .* fliplr(baffleMask);
            transOut{4} = baffleMask .* transIn{4};
        end
    end
end