function out = imc00MN( Pabs, nx, ny, Nbin )

% inject 00 mode
imc0 = IMCIFO( 'ind1', 0, 'ind2', 0, 'withMap', 0, 'PcoatMC13', Pabs, 'PcoatMC2', Pabs );
imc0.lock;

% inject MN mode
imc = IMCIFO( 'ind1', nx, 'ind2', ny, 'withMap', 0, 'PcoatMC13', Pabs, 'PcoatMC2', Pabs );
% copy locked length
imc.props.setPropData( imc0.props.getPropData );

[eta00p, aFSR] = getIMCprop( imc );
eta00p = eta00p / pi;

NRFs = Nbin+1;
out = zeros(NRFs,5);

% idealistic RF frequency of this mode, in units of FSR
RFs = eta00p * (nx + ny);
if mod(nx,2) == 1
    RFs = RFs + 0.5;
end
% pick the smallest frequency
RFs = mod(RFs,1);
% make it Hz by multiplying FSR
RFs = (RFs + 0.003*(-1.5:2/Nbin:0.5)) * aFSR;

for irf = 1:length(RFs)
    
    imc.setRF( RFs(irf) );
    imc.calc( 1e-6, 0 );
    out(irf,1) = RFs(irf);
    out(irf,2) = imc.power('MC2-HRx-o');                 % total power
    out(irf,3) = 1 - imc.HGFrac('MC2-HRx-o',0,0);        % total HOM fraction
    out(irf,4) = abs(imc.HGCoef('MC2-HRx-o', nx, ny))^2; % power of HG(nx,ny)
    out(irf,5) = imc.roundtripLoss;
    
    if mod(irf, 5) == 1
        fprintf(1, '%d th event, power = %d\n', irf, out(irf,2) );
    end
end

function [eta00, FSR] = getIMCprop( imc )
% function [eta00, FSR] = getIMCprop( imc )
% calculate eta00 and FSR of a symmetric triangular IMC
Prop13 = imc.props.findNamedProp( 'Prop13' );
Prop32 = imc.props.findNamedProp( 'Prop32' );
eta00 = imc.props.propList(Prop13).gouy00/2 + imc.props.propList(Prop32).gouy00;
Lcav = imc.props.propList(Prop13).L0/2 + imc.props.propList(Prop32).L0;
FSR = ConstantTK.c / (2*Lcav);
return

