function runKAGRAPRM( varargin )
% runKAGRAPRM( 'power', [1], 'maxLoop', [10], 'absSub', [2e-6*10/100], 'absCoat', [0.5e-6] )

ip = inputParser;
addParameter(ip, 'power',  1,  @(x)isnumeric(x) && x>0); % power of laser
addParameter(ip, 'maxLoop',  10,  @(x)isnumeric(x) && x>=0); % number of iteration finding thermal equilibrium
addParameter(ip, 'absSub',  2e-6*10/100,  @(x)isnumeric(x) && x>=0); % substrate absorption, def = 2ppm/m for 10cm
addParameter(ip, 'absCoat',  0.5e-6,  @(x)isnumeric(x) && x>=0); % coating absorption, def = 0.5ppm
ip.parse(varargin{:})

epsLimit = 1e-5;

% build KAGRA PRM
% This is hot case CR calculation
fprintf(1,'>>> building IFO\n');
prm = KAGRAPRFPM( 'power', ip.Results.power );

fprintf('>>> locking without thermal deformation\n');
% lock IFO
prm.lock(1e-5,0);

% print summary
KAGRAanal(sprintf('CR in IFO without thermal deformation with input power %g\n', ip.Results.power), prm );

fprintf('>>> looping to find thermal equilibrium\n');
% LG00 power in two arms are used to test convergence
PIX0 = abs(prm.LGCoef('ITMX-HR-o',0,0));
PIY0 = abs(prm.LGCoef('ITMY-HR-o',0,0));

for iloop = 1:ip.Results.maxLoop
    prm = onePRMThermal( prm, ip.Results.absSub, ip.Results.absCoat );
    prm.lock(1e-5,0);
    PIX = abs(prm.LGCoef('ITMX-HR-o',0,0));
    PIY = abs(prm.LGCoef('ITMY-HR-o',0,0));
    fprintf(1,'Power changed from %g to %g in X and %g to %g in Y\n', PIX0, PIX, PIY0, PIY);
    epsP = abs(PIX/PIX0-1) + abs(PIY/PIY0-1);
    if epsP < epsLimit
        fprintf(1,'Power change converged (%g) and exits after %d iretation\n\n', epsP, iloop);
        break;
    end
    PIX0 = PIX;
    PIY0 = PIY;
end

KAGRAanal('CR in thermal equilibrium', prm );

fprintf('>>> Calculating SB using the locked state determined by CR\n');
% set RF frequency, units in Hz
prm.setRF(45.188053e6);

% use calc to calculate field without adjusting cavity lengths
prm.calc(1e-4,0, 1e-3);

KAGRAanal('Upper RF', prm );
