% function thermalLens = thermalLensCMSL( model, x, y, thick, Nthick, incAngle, Nnominal, dndt )
%
% Calculate integral of temperature - T_amb along lines from HR to AR (z=0 to z=thick), multiplied by dndt
% For BS, use incAngle = pi/4 for HR to X arm, -pi/4 for HR to SRC
%
function thermalLens = thermalLensCMSL( model, x, y, thick, Nthick, incAngle, Nnominal, dndt )

% default values
if ~exist( 'Nthick', 'var' ) || Nthick == 0
    Nthick = 21;
end
if ~exist( 'incAngle', 'var' )
    incAngle = 0;
end
if ~exist( 'Nnominal', 'var' )
    Nnominal = 1.45;
end
if ~exist( 'dndt' )
    dndt = 8.7e-6;
end

slope = tan( asin( sin( incAngle ) / Nnominal ) );
thermalLens = integrateCMSL( model, 'T-T_amb', x, y, thick, Nthick, slope ) * dndt;
